/*************************************************************************/
/* mie.h                                                                 */
/*                                                                       */
/* mie.h - Mie calculations                                        @ti@  */
/*                                                                       */
/* Author:                                                               */
/*   Bernhard Mayer and Arve Kylling                               @au@  */
/*                                                                       */
/* ----------------------------------------------------------------------*/
/* Copyright (C) 2000 Bernhard Mayer and Arve Kylling                    */
/*                                                                       */
/* This program is free software; you can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation; either version 1, or (at your option)   */
/* any later version.                                                    */
/*                                                                       */
/* This program is distributed in the hope that it will be useful,       */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of        */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the          */
/* GNU General Public License for more details.                          */
/*                                                                       */
/* To obtain a copy of the GNU General Public License write to the       */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,    */
/* USA.                                                                  */
/*-----------------------------------------------------------------------*/
/*************************************************************************/

#ifndef __miecalc_h
#define __miecalc_h

#include <stdio.h>

typedef struct {
  float re;
  float im;
} mie_complex;

#define USER     1
#define ICE      2
#define WATER    3

#define MIEV0    1
#define BHMIE    2

/* Wavelength input grid STRUCTURE */

typedef struct {
  double   wl_start;
  double   wl_end;
  double   wl_step;
} mie_wl_inp_struct;

/* Wavelength output grid STRUCTURE */

typedef struct {
  int      nlambda;
  double*  lambda;
} mie_wl_out_struct;

typedef struct {
  mie_complex *s1, *s2;  /* Mie scattering amplitudes */
  int         anyang;    /* If true xmu any angles may be given in xmu */
  int         ipolzn;    /* Type of pmom */
  int         momdim;    /* First dimension of pmom */
  int         nmom;      /* Highest Legendre moment */
  int         numang;    /* No. of angles in xmu */
  int         perfct;    /* If true assume refractive index is infinite */
  int         prnt[2];   /* Print flag */
  float       mimcut;    /* Imaginary component zeroed below mimcut */
  float       *xmu;      /* Cosine of angles for s1 and s2 */
} mie_inp_struct;

typedef struct {
  mie_complex crefin;    /* Complex refractive index */
  mie_complex sback;     /* Back scattering amplitude */
  mie_complex sforw;     /* Forward scattering amplitude */
  mie_complex tback[2];  /* See miev.doc */
  mie_complex tforw[2];  /* See miev.doc */
  float       qext;      /* Extinction efficiency factor */
  float       qsca;      /* Scattering efficiency factor */
  float       qback;     /* Backscattering efficiency factor */
  float       gsca;      /* Asymmetry factor */
  float       gqsc;      /* Asymmetry times scattering efficiency factor */
  float       spike;     /* Magnitude of smallest denominator of Mie coefficients */
  float       xx;        /* Mie size parameter */
  float       **pmom;    /* Phase function moments */
} mie_out_struct;


int mie_calc (mie_inp_struct input, mie_out_struct *output, 
	      int program, int medium, mie_complex crefin, 
	      float wavelength, float radius, 
	      float temperature, mie_complex *ref);

int mie_calc_sizedist (mie_inp_struct input, mie_out_struct *output, 
		       int program, int medium, mie_complex crefin, 
		       float wavelength, float temperature, 
		       double *x_size, double *y_size, int n_size,
		       double *beta, double *omega, double *g);

int phase_function (float *moment, int L);
int cumulative_probability (float *moment, int L);

int read_mie_table (char *filename, 
		    double *r0, double *dr, int *nreff, 
		    double *wavelen, double *nre, double *nim, 
		    double **extinc, double **albedo, int **nleg, 
		    float ***legen);



#endif



